@@ -171,7 +171,15 @@ fn attempt_assignment_tactics(
             let shape = shape.reset().increment_additional_indent();
 
             // As we know that there is only a single element in the list, we can extract it to work with it
-            let expression = format_expression(ctx, expression, shape);
+            // Format the expression given - if it contains comments, make sure to hang the expression
+            // Ignore the leading comments though (as they are solved by hanging at the equals), and the
+            // trailing comments, as they don't affect anything
+            let expression =
+                if trivia_util::expression_contains_inline_comments(&strip_trivia(expression)) {
+                    hang_expression(ctx, expression, shape, None)
+                } else {
+                    format_expression(ctx, expression, shape)
+                };
 
             // We need to take all the leading trivia from the expr_list
             let (expression, leading_comments) =
@@ -322,14 +330,14 @@ fn format_local_no_assignment(
         name_list = name_list.update_trailing_trivia(FormatTriviaType::Append(trailing_trivia))
     }
 
-    let local_assignment = LocalAssignment::new(name_list)
-        .with_local_token(local_token)
-        .with_equal_token(None)
-        .with_expressions(Punctuated::new());
-
+    let local_assignment = LocalAssignment::new(name_list);
     #[cfg(feature = "luau")]
     let local_assignment = local_assignment.with_type_specifiers(type_specifiers);
+
     local_assignment
+        .with_local_token(local_token)
+        .with_equal_token(None)
+        .with_expressions(Punctuated::new())
 }
 
 pub fn format_local_assignment(
@@ -421,12 +429,12 @@ pub fn format_local_assignment(
         // Add necessary trivia
         let expr_list = expr_list.update_trailing_trivia(FormatTriviaType::Append(trailing_trivia));
 
-        let local_assignment = LocalAssignment::new(name_list)
-            .with_local_token(local_token)
-            .with_equal_token(Some(equal_token))
-            .with_expressions(expr_list);
+        let local_assignment = LocalAssignment::new(name_list);
         #[cfg(feature = "luau")]
         let local_assignment = local_assignment.with_type_specifiers(type_specifiers);
         local_assignment
+            .with_local_token(local_token)
+            .with_equal_token(Some(equal_token))
+            .with_expressions(expr_list)
     }
 }
